@@ -270,6 +270,30 @@ class cognet:
         else:
             return qsample(sample,self.qnet,steps)
 
+    def random_sample(self,
+                      df=None):
+        '''compute a random sample from the underlying distributions of the dataset, by column.
+        
+        
+        Args:
+          df (pandas.DataFrame): Desired data to take random sample of. Defaults to None, in which case qnet samples are used.
+          
+        Returns:
+          return_df (pd.DataFrame): Random sample drawn from underlying distribution of each column.
+        '''
+        # check if a new dataset was inputted
+        if df is None:
+            samples_ = self.samples
+        else:
+            samples_ = df
+
+        # take random sample from each of the columns based on their distribution
+        return_df = pd.DataFrame()
+        for col in samples_.columns:
+            return_df[col] = samples_[col].sample(n=1)
+            
+        return return_df
+    
     def set_poles(self,
                   POLEFILE,
                   pole_1,
@@ -278,7 +302,7 @@ class cognet:
                   mutable=False,
                   VERBOSE=False,
                   restrict=True,
-                  nsamples = False,
+                  nsamples = None,
                   random=False):
         '''set the poles and samples such that the samples contain features in poles
 
@@ -313,9 +337,11 @@ class cognet:
                 cols = [x for x in self.poles.columns if x in self.samples.columns]
                 self.samples=self.samples[cols]
                 self.restricted = True
-              
-            # if poles had been restricted before, unrestrict it and set original
-            elif self.restricted:
+                self.samples = pd.concat([self.features,self.samples], axis=0).replace("nan","").fillna('')
+                self.samples_as_strings = self.samples[self.cols].fillna('').values.astype(str)[:]
+                
+            # if restrict==False, unrestrict it and set original
+            else:
                 self.restricted = False
                 self.samples = self.all_samples
                 if self.nsamples is not None:
@@ -326,10 +352,7 @@ class cognet:
                 for x in self.poles.columns:
                     if x not in self.samples.columns:
                         invalid_count += 1
-                        self.samples[x]=''
-
-            self.samples = pd.concat([self.features,self.samples], axis=0).fillna('')
-            self.samples_as_strings = self.samples[self.cols].fillna('').values.astype(str)[:]
+                        #self.samples[x]=''
             
             if mutable:
                 self.mutable_vars=[x for x in self.cols if x in self.poles.columns]
@@ -420,8 +443,8 @@ class cognet:
         return qdistance(sample1, sample2, self.qnet, self.qnet)
     
     def __distfunc(self, 
-                x, 
-                y):
+                   x, 
+                   y):
         '''Compute distance between two samples
 
         Args:
@@ -481,7 +504,7 @@ class cognet:
             # format and save resulting dict, and tranpose symmetrical distance matrix
             result = result.to_numpy()
             result = pd.DataFrame(np.maximum(result, result.transpose()))
-            result.to_csv(outfile, index=None)
+            result.to_csv(outfile, index=None, header=None)
         else:
             raise ValueError("load data first!")
         
@@ -1027,6 +1050,7 @@ class cognet:
     
     def dmat_filewriter(self,
                         QNETPATH,
+                        mpi_path="mpi_tmp/",
                         pyfile="cognet_qdistmatrix.py",
                         MPI_SETUP_FILE="mpi_setup.sh",
                         MPI_RUN_FILE="mpi_run.sh",
@@ -1061,7 +1085,7 @@ class cognet:
                 self.set_nsamples(num_samples)
             
             # init and make tmp dir 
-            tmp_path = "mpi_tmp/"
+            tmp_path = mpi_path
             if not os.path.exists(tmp_path):
                 os.makedirs(tmp_path)
             
